<?php
namespace XenGenTr\ForumistatistikPlus\Pub\Controller;

class Licenses extends \XF\Pub\Controller\AbstractController
{
    public function actionIndex(\XF\Mvc\ParameterBag $params)
    {
        if($params->user_id) {
            return $this->rerouteController("XenGenTr\\ForumistatistikPlus\\Pub\\Controller\\Licenses", "Licenses", $params);
        }
        $visitor = \XF::visitor();
        if($visitor->user_id) {
            $params->user_id = $visitor->user_id;
            return $this->rerouteController("XenGenTr\\ForumistatistikPlus:Licenses", "Licenses", $params);
        }
        $memberStat = $this->em()->findOne("XF:MemberStat", ["member_stat_key" => "lisans_yok"]);
        if($memberStat && $memberStat->canView()) {
            return $this->redirectPermanently($this->buildLink("members", NULL, ["key" => $memberStat->member_stat_key]));
        }
        return $this->redirect($this->buildLink("sabit-konular"));
    }
    public function actionLicenses(\XF\Mvc\ParameterBag $params)
    {
        $user = $this->assertRecordExists("XF:User", $params->user_id);
        $licenseRepo = $this->repository("XenGenTr\\ForumistatistikPlus:License");
        $finder = $licenseRepo->findLicensesByUser($user->user_id);
        $this->assertCanonicalUrl($this->buildLink("sabit-konular/licenses", $user));
        $licenses = $finder->fetch();
        $licenses = $licenses->filterViewable();
        $canInlineMod = false;
        foreach ($licenses as $license) {
            if($license->Product->canUseInlineModeration()) {
                $canInlineMod = true;
                $viewParams = ["user" => $user, "tree" => $licenseRepo->createLicenseTree($licenses), "canInlineMod" => false];
                return $this->view("XenGenTr\\ForumistatistikPlus:Licenses\\View", "sabit_konular_liste", $viewParams);
            }
        }
    }
    public function actionView(\XF\Mvc\ParameterBag $params)
    {
        $extraWith = ["User", "Product", "Product.LatestVersion"];
        $license = $this->assertValidLicense($params->license_key, $extraWith);
        if($this->isPost()) {
            if(!$license->canEdit($error)) {
                throw $this->exception($this->noPermission($error));
            }
            if($license->Product->isAddOn()) {
                if(!$license->parent_license_id) {
                    $parent = $this->assertValidLicense($this->filter("parent_license", "str"), $extraWith);
                    foreach ($parent->Product->Children as $child) {
                        if($child->sabitkonu_id == $license->Product->sabitkonu_id) {
                            $license->parent_license_id = $parent->license_id;
                            if(!$license->parent_license_id) {
                                throw $this->exception($this->error(\XF::phrase("sabit_konular_dogrula")));
                            }
                            $fieldSet = $license->license_fields;
                            $fieldDefinition = $fieldSet->getDefinitionSet()->filterEditable($fieldSet, "user");
                            $licenseFieldsShown = array_keys($fieldDefinition->getFieldDefinitions());
                            if($licenseFieldsShown) {
                                $fieldSet->bulkSet($parent->license_fields_, $licenseFieldsShown);
                            }
                            $license->save();
                        }
                    }
                }
            } else {
                $fieldSet = $license->license_fields;
                $fieldDefinition = $fieldSet->getDefinitionSet()->filterEditable($fieldSet, "user");
                $licenseFieldsShown = array_keys($fieldDefinition->getFieldDefinitions());
                if($licenseFieldsShown) {
                    $fieldSet->bulkSet($this->filter("license_fields", "array"), $licenseFieldsShown);
                }
                $license->save();
                if($license->Children) {
                    foreach ($license->Children as $child) {
                        $fieldSet = $child->license_fields;
                        $fieldDefinition = $fieldSet->getDefinitionSet()->filterEditable($fieldSet, "user");
                        $licenseFieldsShown = array_keys($fieldDefinition->getFieldDefinitions());
                        if($licenseFieldsShown) {
                            $fieldSet->bulkSet($license->license_fields_, $licenseFieldsShown);
                        }
                        $child->save();
                    }
                }
            }
            return $this->redirect($this->buildLink("sabit-konular/licenses", $license->User));
        }
        $errors = $parentLicenses = [];
        $isValid = $license->hasValidLicenseFields("user", $errors);
        $warnings = [];
        if($license->Product->isAddOn()) {
            if(!$license->parent_license_id) {
                $isValid = false;
                $errors = [\XF::phraseDeferred("sabit_konular_tamamla")];
                $licenseRepo = $this->repository("XenGenTr\\ForumistatistikPlus:License");
                $parentLicenses = $licenseRepo->findOtherLicensesByUser($license)->where("parent_license_id", 0)->fetch()->filterViewable()->filter(function (\XenGenTr\ForumistatistikPlus\Entity\License $parent) use($parent) {
                    foreach ($parent->Children as $childLicense) {
                        if($childLicense->sabitkonu_id == $license->sabitkonu_id) {
                            return NULL;
                        }
                    }
                    foreach ($parent->Product->Children as $childProduct) {
                        if($childProduct->sabitkonu_id == $license->sabitkonu_id) {
                            return $parent;
                        }
                    }
                    return NULL;
                })->pluck(function (\XenGenTr\ForumistatistikPlus\Entity\License $license) {
                    $title = $license->title;
                    $fieldSet = $license->license_fields;
                    $fieldDefinition = $fieldSet->getDefinitionSet()->filterGroup("list");
                    $definitions = $fieldDefinition->getFieldDefinitions();
                    foreach ($definitions as $fieldDefinition) {
                        $value = $fieldSet->getFieldValue($fieldDefinition["field_id"]);
                        $title .= " - " . ($value ?: "N/A");
                    }
                    return [$license->license_key, $title];
                });
            } elseif(!$isValid) {
                $errors = [\XF::phraseDeferred("lisans_bilgisi")];
            }
        }
        $hasDownload = $hasDiscussion = false;
        $downloads = [];
        if($license->Product->LatestVersion) {
            $visitor = \XF::visitor();
            $downloadRepo = $this->repository("XenGenTr\\ForumistatistikPlus:Download");
            $downloadFinder = $downloadRepo->findSabitKonular($license->Product)->with(["Discussion", "Discussion.Forum", "Discussion.Forum.Node", "Discussion.Forum.Node.Permissions|" . $visitor->permission_combination_id])->limit($this->options()->sabitKonularPerPage);
            $downloads = $downloadFinder->fetch();
            $hasDownload = $hasDiscussion = false;
            foreach ($downloads as $download) {
                if($download->canDownload($license)) {
                    $hasDownload = true;
                }
                if($download->hasViewableDiscussion()) {
                    $hasDiscussion = true;
                }
            }
        }
        $viewParams = ["license" => $license, "isValid" => $isValid, "validationErrors" => $errors, "validationWarnings" => $warnings, "parentLicenses" => $parentLicenses, "downloads" => $downloads, "hasDownload" => $hasDownload, "hasDiscussion" => $hasDiscussion];
        return $this->view("XenGenTr\\ForumistatistikPlus:Licenses\\Info", "lisans_gor", $viewParams);
    }
    protected function assertLicenseExists($id, $with = NULL, $phraseKey = NULL)
    {
        return $this->assertRecordExists("XenGenTr\\ForumistatistikPlus:License", $id, $with, $phraseKey);
    }
}

?>