<?php
namespace XenGenTr\LisansKontrolSistemi\Authentication;

class Core extends AbstractAuth
{
    protected $hashFunc = "sha1";
    protected function setup()
    {
        if(!empty($this->data["hashFunc"])) {
            $this->hashFunc = $this->data["hashFunc"];
        } else {
            $this->hashFunc = extension_loaded("hash") ? "sha256" : "sha1";
        }
    }
    protected function createHash($hash)
    {
        switch ($this->hashFunc) {
            case "sha256":
                return hash("sha256", $hash);
                break;
            case "sha1":
                return sha1($hash);
                break;
            default:
                throw new \InvalidArgumentException("Unknown hash type");
        }
    }
    protected function getApiWordHash($api, $salt)
    {
        return $this->createHash($this->createHash($api) . $salt);
    }
    public function generate($api)
    {
        if(!is_string($api) || $api === "") {
            return false;
        }
        $salt = $this->createHash(\XF::generateRandomString(20, true));
        return ["hash" => $this->getApiWordHash($api, $salt), "salt" => $salt, "hashFunc" => $this->hashFunc];
    }
    public function authenticate($userId, $api)
    {
        if(!is_string($api) || $api === "" || empty($this->data)) {
            return false;
        }
        $userHash = $this->getApiWordHash($api, $this->data["salt"]);
        return hash_equals($this->data["hash"], $userHash);
    }
    public function getAuthenticationName()
    {
        return "XF:Core";
    }
}

?>