<?php
namespace XenGenTr\LisansKontrolSistemi\Behavior;

class RedisCache extends \Doctrine\Common\Cache\RedisCache
{
    private $serializer;
    const SERIALIZER_PHP = 1;
    const SERIALIZER_IGBINARY = 2;
    public function setRedis(\Redis $redis)
    {
        parent::setRedis($redis);
        $this->getRedis()->setOption(\Redis::OPT_SERIALIZER, \Redis::SERIALIZER_NONE);
        $this->serializer = $this->getSerializerValue();
    }
    public function getSerializer()
    {
        return $this->serializer;
    }
    protected function doFetch($id)
    {
        $value = parent::doFetch($id);
        return $value !== false ? $this->unserialize($value) : false;
    }
    protected function doFetchMultiple(array $keys)
    {
        $redis = $this->getRedis();
        $values = $redis->mget($keys);
        if($values === false) {
            return [];
        }
        $fetchedItems = array_combine($keys, $values);
        $foundItems = [];
        foreach ($fetchedItems as $key => $value) {
            if($value !== false) {
                $foundItems[$key] = $this->unserialize($value);
            }
        }
        return $foundItems;
    }
    protected function doSave($id, $data, $lifeTime = 0)
    {
        $redis = $this->getRedis();
        return 0 < $lifeTime ? $redis->setex($id, $lifeTime, $this->serialize($data)) : $redis->set($id, $this->serialize($data));
    }
    protected function doDelete($id)
    {
        $redis = $this->getRedis();
        return 0 <= $redis->del($id);
    }
    protected function serialize($value)
    {
        return $this->serializer === self::SERIALIZER_IGBINARY ? igbinary_serialize($value) : serialize($value);
    }
    protected function unserialize($value)
    {
        return $this->serializer === self::SERIALIZER_IGBINARY ? igbinary_unserialize($value) : unserialize($value);
    }
    protected function getSerializerValue()
    {
        if(defined("Redis::SERIALIZER_IGBINARY") && extension_loaded("igbinary")) {
            return self::SERIALIZER_IGBINARY;
        }
        return self::SERIALIZER_PHP;
    }
}

?>