<?php
namespace XenGenTr\LisansKontrolSistemi\PreRegAction;

abstract class AbstractHandler
{
    protected $actionType;
    public function __construct($actionType)
    {
        $this->actionType = $actionType;
    }
    public abstract function getContainerContentType();
    public abstract function getDefaultActionData() : array;
    protected abstract function canCompleteAction($action, \XF\Mvc\Entity\Entity $containerContent, \XF\Entity\User $newUser);
    protected abstract function executeAction(\XF\Entity\PreRegAction $action, \XF\Mvc\Entity\Entity $containerContent, \XF\Entity\User $newUser);
    protected abstract function sendSuccessAlert(\XF\Entity\PreRegAction $action, \XF\Mvc\Entity\Entity $containerContent, \XF\Entity\User $newUser, \XF\Mvc\Entity\Entity $executeContent);
    public function saveAction($PreRegAction, $containerContent, $actionData)
    {
        if($containerContent->getEntityContentType() !== $this->getContainerContentType()) {
            throw new \InvalidArgumentException("Passed in container entity does not match the expected content type");
        }
        $action = \XF::em()->create("XF:PreRegAction");
        $action->action_class = $this->actionType;
        $action->content_id = $containerContent->getEntityId();
        $action->action_data = $actionData;
        $action->ip_address = \XF\Util\Ip::convertIpStringToBinary(\XF::app()->request()->getIp());
        $this->setupActionEntity($containerContent, $action, $actionData);
        $action->save();
        return $action;
    }
    protected function setupActionEntity(\XF\Mvc\Entity\Entity $containerContent, \XF\Entity\PreRegAction $action, $actionData)
    {
    }
    public function completeAction(\XF\Entity\PreRegAction $action, \XF\Entity\User $newUser)
    {
        if(!$newUser->canCompletePreRegAction()) {
            $action->delete();
        } else {
            $containerContent = $this->getContainerContent($action->content_id);
            if($containerContent) {
                $executeContent = \XF::asVisitor($newUser, function () use($action) {
                    if(!$this->canCompleteAction($action, $containerContent, $newUser)) {
                        return NULL;
                    }
                    return $this->executeAction($action, $containerContent, $newUser);
                });
            } else {
                $executeContent = NULL;
            }
            if($executeContent) {
                $this->sendSuccessAlert($action, $containerContent, $newUser, $executeContent);
            } else {
                $this->sendFailureAlert($action, $newUser);
            }
            $action->delete();
            return $executeContent;
        }
    }
    protected function sendFailureAlert(\XF\Entity\PreRegAction $action, \XF\Entity\User $newUser)
    {
        $alertRepo = \XF::repository("XF:UserAlert");
        $alertRepo->alertFromUser($newUser, NULL, "user", $newUser->user_id, "pre_reg_failed", ["welcome" => $action->isForNewUser()]);
    }
    public function getContainerEntityWith()
    {
        return [];
    }
    public function getContainerContent($id)
    {
        return \XF::app()->findByContentType($this->getContainerContentType(), $id, $this->getContainerEntityWith());
    }
    public function getActionType()
    {
        return $this->actionType;
    }
    public function renderApprovalQueueInfo(\XF\Entity\PreRegAction $preRegAction)
    {
        $containerContent = $preRegAction->ContainerContent;
        if(!$containerContent) {
            return "";
        }
        $data = $this->getStructuredContentData($preRegAction, $containerContent);
        if(!$data) {
            return "";
        }
        $data = array_replace(["title" => NULL, "title_link" => NULL, "content_title" => NULL, "text" => NULL, "bb_code" => NULL], $data);
        $templateParams = ["preRegAction" => $preRegAction, "containerContent" => $containerContent, "details" => $data];
        return \XF::app()->templater()->renderTemplate($this->getApprovalQueueTemplate(), $templateParams);
    }
    public function getContentForSpamCheck($preRegAction)
    {
        $containerContent = $preRegAction->ContainerContent;
        if(!$containerContent) {
            return "";
        }
        $data = $this->getStructuredContentData($preRegAction, $containerContent);
        if(!$data) {
            return "";
        }
        $data = array_replace(["content_title" => NULL, "text" => NULL, "bb_code" => NULL], $data);
        if($data["content_title"]) {
            $content = $data["content_title"] . "\n\n";
        } else {
            $content = "";
        }
        if($data["text"]) {
            $content .= $data["text"] . "\n\n";
        }
        if($data["bb_code"]) {
            $content .= $data["bb_code"] . "\n\n";
        }
        return trim($content);
    }
    protected function isFlooding($checkAction, \XF\Entity\User $newUser = NULL, $floodingLimit)
    {
        if($newUser->hasPermission("general", "bypassFloodCheck")) {
            return false;
        }
        $floodChecker = \XF::service("XF:FloodCheck");
        $timeRemaining = $floodChecker->checkFlooding($checkAction, $newUser->user_id, $floodingLimit);
        if($timeRemaining) {
            return true;
        }
        return false;
    }
    protected function getStructuredContentData($preRegAction, \XF\Mvc\Entity\Entity $containerContent)
    {
        return [];
    }
    protected function getApprovalQueueTemplate()
    {
        return "public:pre_reg_action_approval_queue";
    }
}

?>