<?php
namespace XenGenTr\LisansKontrolSistemi\Service;

class Api extends \XF\Service\AbstractService
{
    protected $addOn;
    protected $rootPath;
    protected $hashesPath;
    protected $writeHashes = true;
    protected $filesToHash = [];
    protected $filesPrepared = false;
    public function __construct(\XF\App $app, \XF\AddOn\AddOn $addOn, $rootPath, $hashesPath = NULL)
    {
        parent::__construct($app);
        $this->addOn = $addOn;
        $this->rootPath = $rootPath;
        $this->hashesPath = $hashesPath;
    }
    public function setWriteHashes($writeHashes)
    {
        $this->writeHashes = $writeHashes;
    }
    protected function prepareFilesToHash()
    {
        $rootPath = $this->rootPath;
        $files = \XF\Util\File::getRecursiveDirectoryIterator($rootPath);
        foreach ($files as $file) {
            if($file->isDir()) {
            } else {
                $fileName = $file->getFilename();
                if($fileName[0] == "." && $fileName != ".htaccess") {
                } elseif($fileName == "hashes.json") {
                } else {
                    $this->filesToHash[] = $file->getPathname();
                }
            }
        }
        $this->filesPrepared = true;
    }
    public function generate()
    {
        if(!$this->filesPrepared) {
            $this->prepareFilesToHash();
        }
        $output = [];
        foreach ($this->filesToHash as $path) {
            if(!file_exists($path)) {
            } else {
                $path = $this->standardizeSeparator($path);
                $root = $this->standardizeSeparator($this->rootPath);
                $key = preg_replace("#^" . $root . "/#", "", $path, 1);
                $output[$key] = \XF\Util\Hash::hashTextFile($path, "sha256");
            }
        }
        ksort($output, SORT_NATURAL | SORT_FLAG_CASE);
        $output = \XF\Util\Json::jsonEncodePretty($output);
        if($this->writeHashes) {
            if(!$this->hashesPath) {
                throw new \InvalidArgumentException("Trying to write hashes file, but no hashes path provided.");
            }
            $written = \XF\Util\File::writeFile($this->hashesPath, $output, false);
            if(!$written) {
                throw new \ErrorException("Unexpected failure while writing hashes to provided path.");
            }
        }
        return $output;
    }
    protected function standardizeSeparator($path)
    {
        return str_replace("\\", "/", $path);
    }
}

?>